from glob import glob
import subprocess
import re
import os

from . depends import provides, requires
from . output import warning, error
import checks

class BinaryPackages(object):
    arches = []
    changes_file = None
    debs = []
    extracted_deb_locations = []
    deb_contents = []

    def __init__(self, path, sourcepackage):
        self.path = path
        self.sp = sourcepackage
    
    @provides('arches')
    def find_arches(self):
        self.arches += [subprocess.check_output(['dpkg', '--print-architecture']).strip()]
        for arch in subprocess.check_output(
                ['dpkg', '--print-foreign-architectures']).split("\n"):
            if arch:
                self.arches += [arch.strip()]

    @provides('changes_file')
    def find_changes_file(self):
        if not self.sp.package_name:
            error("Could not find out package name. Can't do binary package "
                  "tests.")
            return
        changes_files = glob("%s/%s*.changes" % (self.path, 
                                                 self.sp.package_name))
        if not changes_files:
            error("Could not find .changes file. Can't do binary package "
                  "tests.")
            return
        for changes_file in changes_files:
            if filter(lambda arch: changes_file.endswith("_%s.changes" % arch), 
                      self.arches):
                self.changes_file = changes_file
                break
        if not self.changes_file:
            error("Could not find .changes file. Can't do binary package "
                  "tests.")

    @provides('debs')
    @requires('changes_file')
    def find_debs(self):
        if not self.changes_file:
            return None
        f = open(self.changes_file, "r")
        content = f.read()
        f.close()
        arch_regex = "(%s)" % ("|".join(self.arches+["all"]))
        regex = "(\S+?\_\S+?\_%s\.deb)" % arch_regex
        self.debs = set(map(lambda a: a[0], 
                            re.findall(regex, content)))
        if not self.debs:
            error("No packages found.")

    @provides('deb_contents')
    @requires('debs')
    def find_deb_contents(self):
        if not self.debs:
            return None
        for deb in self.debs:
            deb_file = os.path.join(os.path.dirname(self.changes_file),
                                    deb)
            tmp_path = os.path.join("/tmp",
                                    deb.split("_")[0]+"-test")
            if os.path.exists(tmp_path):
                subprocess.call(["rm", "-r", tmp_path])
            subprocess.call(["dpkg", "-x", deb_file, tmp_path])
            self.extracted_deb_locations += [ tmp_path ]
        for extracted_deb_location in self.extracted_deb_locations:
            for path, dirs, files in os.walk(extracted_deb_location):
                for filename in files:
                    self.deb_contents += [os.path.join(path, filename)]
        prefix = os.path.commonprefix(self.deb_contents)
        self.deb_contents = map(lambda a: a.split(prefix)[1],
                                self.deb_contents)

    @requires('deb_contents')
    def test_translations_outside_opt(self):
        for filename in self.deb_contents:
            if "usr/share/locale" in filename and \
               "opt/" not in filename:
                error("This package contains translations, which are not "
                      "shipped in /opt. This issue needs to be fixed. Look "
                      "at XXX for an example how do fix it.")
                break

    @requires('deb_contents')
    def test_incorrect_lens_files(self):
        if not self.sp.is_lens:
            return None
        offending_files = filter(lambda a: \
                (a.endswith(".service") or a.endswith(".lens")) and \
                not os.path.basename(a).startswith("extras-unity-"), self.deb_contents)
        if offending_files:
            error("The following file names should start with "
                  "'extras-unity-':",
                  offending_files)
    
    @requires('deb_contents')
    def test_files_in_opt(self):
        if not self.deb_contents:
            return
        correct_opt_path = "opt/extras.ubuntu.com/" + self.sp.package_name
        doc_path = "usr/share/doc/" + self.sp.package_name
        exceptions = [
            ".lens", 
            ".desktop", 
            ".service",
            doc_path,
                     ]
        files_in_opt = filter(lambda filename: correct_opt_path in filename,
                              self.deb_contents)
        files_not_in_opt = filter(lambda filename: correct_opt_path not in filename,
                                  self.deb_contents)
        if not files_in_opt:
            error("The package seems to contain no files in '/%s' which "
                  "indicates a problem with the installation of files." % \
                          correct_opt_path)
        other_std_files = checks.find_offending_items(files_not_in_opt, exceptions)
        if other_std_files:
            error("These files are outside %s and shouldn't be: " % correct_opt_path,
                  other_std_files)

        rtupdate_file = "usr/share/python/runtime.d/%s.rtupdate" % self.sp.package_name
        if rtupdate_file in other_std_files:
            warning("Unallowed file %s was found in the package, which is "
                    "created by dh_python2. We still have to make a decision "
                    "about how to proceed with the issue, follow the progress "
                    "on https://bugs.launchpad.net/ubuntu/+source/python-distutils-extra/+bug/1021207" % \
                          rtupdate_file)

        crashdb_file = "etc/apport/crashdb.conf.d/%s-crashdb.conf" % self.sp.package_name
        if crashdb_file in other_std_files:
            error("Unallowed file %s was found in the package, which is "
                  "due to LP: #1020502. To work around this issue, have "
                  "a look at "
                  "http://bazaar.launchpad.net/~dholbach/ubuntu-app-reviews/harvestwidget/revision/58" % \
                          crashdb_file)
