import os

from . depends import provides, requires
from . output import error, warning
import checks

class SourcePackage(object):
    package_name = None
    source_files = []
    is_lens = False
    is_distutils = False
    has_copyright = False
    has_debian_control = False

    @provides(['cwd', 'debian_dir'])
    def __init__(self):
        self.cwd = os.getcwd()
        self.debian_dir = os.path.join(self.cwd, "debian")

    @provides('package_name')
    def find_package_name(self):
        changelog = os.path.join(self.debian_dir, "changelog")
        if not os.path.exists(changelog):
            return None
        f = open(changelog, "r")
        line = f.readline()
        f.close()
        self.package_name = line.split()[0]

    @provides('source_files')
    def find_source_files(self):
        if self.source_files:
            return None
        for path, dirs, files in os.walk(self.cwd):
            for filename in files:
                self.source_files += [os.path.join(path, filename)]
        prefix = os.path.commonprefix(self.source_files)
        self.source_files = map(lambda a: a.split(prefix)[1],
                                self.source_files)
        self.source_files = filter(lambda a: not a.startswith(".bzr") and \
                                             not a.startswith("debian"),
                                   self.source_files)

    @provides('is_lens')
    @requires('source_files')
    def find_out_if_this_is_a_lens(self):
        if filter(lambda a: a.endswith(".service") or a.endswith(".lens"),
                  self.source_files):
            self.is_lens = True

    @provides('is_distutils')
    def find_out_if_this_is_distutils(self):
        if os.path.exists(os.path.join(self.cwd, 'setup.py')):
            self.is_distutils = True

    @requires('debian_dir')
    def test_has_debian_dir(self):
        if not os.path.exists(self.debian_dir):
            warning("This submission seems not to be packaged yet. You might "
                    "want to reply with something along these lines:"
                    "---"
                    "Thanks for your submission! We've created a simple "
                    "walk-through guide in how to package an application to "
                    "meet the requirements for this submission process."
                    ""
                    "https://wiki.ubuntu.com/AppReviewBoard/Submissions/PackageQuickStart"
                    "Please follow these steps for your application. If the "
                    "instructions are unclear at any point, it would help us "
                    "(and other developers) if you ask us questions about "
                    "what you need to do, so we can improve the guide. You "
                    "can contact us by submitting a reply to the website, on "
                    "the app-review-board@lists.ubuntu.com mailing list, or "
                    "#ubuntu-app-devel IRC channel on Freenode.")

    @provides('has_copyright')
    @requires('debian_dir')
    def test_has_copyright(self):
        copyright_file = os.path.join(self.debian_dir, "copyright")
        if checks.file_is_missing_or_empty(copyright_file):
            error("debian/copyright is missing or empty. Please have a look "
                  "at http://dep.debian.net/deps/dep5/ to find out how to fix "
                  "this.")
        else:
            self.has_copyright = True

    @requires('has_copyright')
    def test_uses_dep5(self):
        copyright_file = os.path.join(self.debian_dir, "copyright")
        first_line = open(copyright_file, "r").readline()
        values = [a.strip() for a in first_line.split()]
        if values[0] != "Format:" or \
           values[1] not in ["http://dep.debian.net/deps/dep5",
                             "http://dep.debian.net/deps/dep5/",
                             "http://www.debian.org/doc/packaging-manuals/copyright-format/1.0",
                             "http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/"]:
            warning("This package does not use DEP-5, which provides a "
                    "machine-readable debian/copyright file. You might want "
                    "to review "
                    "http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/ "
                    "to find out how to accomplish it. "
                    "http://bazaar.launchpad.net/~dholbach/ubuntu-app-reviews/harvestwidget/view/head:/debian/copyright "
                    "has a live example for a simple app.")

    @provides('has_debian_control')
    @requires('debian_dir')
    def test_has_debian_control(self):
        control_file = os.path.join(self.debian_dir, "control")
        if checks.file_is_missing_or_empty(control_file):
            error("debian/control seems to be missing or empty.")
        else:
            self.has_debian_control = True

    @requires('has_debian_control')
    def test_uses_deprecated_python_installation(self):
        control_file = os.path.join(self.debian_dir, "control")
        f = open(control_file, "r")
        contents = f.read()
        f.close()
        if 'python-support' in contents or \
           'python-central' in contents:
            warning("This package uses a deprecated python installation "
                    "tool. You might want to have a look at "
                    "http://wiki.debian.org/Python/TransitionToDHPython2 "
                    "to find out how to transition to dh_python2.")

    @requires('source_files')
    def test_is_lightweight_app(self):
        source_file_extensions = [ ".py", ".rb", ".vala", ".c", ".h", ".cpp", 
                ".cc", ".hh", ".pm", ".java", ".js", ".pl" ]
        number_of_max_source_files = 30
        number_of_max_lines_of_code = 15000
        source_code_files = filter(lambda a: os.path.splitext(a)[1] in source_file_extensions,
                                   self.source_files)
        loc = sum(map(lambda a: len(open(a).readlines()), source_code_files))
        if len(source_code_files) >= number_of_max_source_files or \
           loc >= number_of_max_lines_of_code:
            warning("This app might be too big to be reviewed by the ARB. "
                    "It has %s source files and %s lines of code. "
                    "This might serve as reply to the app submitter: "
                    "We are sorry to inform you that this app is outside the "
                    "scope of the App Review Board. Our focus is on "
                    "lightweight apps. We are generally looking for the kind "
                    "of apps which could be reviewed for functionality and "
                    "security in about an hour reading through the code. "
                    "https://wiki.ubuntu.com/UbuntuDevelopment/NewPackages "
                    "might suit you better." % \
                            (len(source_code_files), loc))

    @requires('debian_dir')
    def test_has_maintainer_scripts(self):
        offending_files = []
        prefixes = [ "post", "pre" ]
        suffixes = [ "rm", "inst" ]
        for prefix in prefixes:
            for suffix in suffixes:
                which = prefix+suffix
                for filename in [ "debian/%s" % which, 
                        "debian/%s.%s"% (self.package_name, which) ]:
                    if os.path.exists(filename):
                        offending_files += [ filename ]
        if offending_files:
            error("This app includes maintainer scripts. The scope of apps "
                  "is for them to be small and unintrusive, so the use of "
                  "maintainer scripts is not allowed. These files should "
                  "not be used:", offending_files)

    @requires('has_debian_control')
    def test_conflicts_replaces_breaks(self):
        control_file = os.path.join(self.debian_dir, "control")
        f = open(control_file, "r")
        contents = f.readlines()
        f.close()
        if filter(lambda a: a.startswith("Breaks:") or \
                            a.startswith("Conflicts:") or \
                            a.startswith("Replaces:"), contents):
            warning("This app seems to specify a Breaks or Conflicts or "
                    "Replaces in debian/control. As apps are supposed to be "
                    "self-contained without relationships to other packages "
                    "this might be a mistake or a problem.")
